
butterfly.plot = function(data, main, filename,score_range){
	data$m_pdf = data$match_count/sum(data$match_count)
	data$n_pdf = data$nonmatch_count/sum(data$nonmatch_count)
	
	rgx = max(max(data$m_pdf),max(data$n_pdf))
	mnx = -rgx
	mxx =  rgx
	mny = min(data$min)
	mxy = max(data$max)
	
	vendor = data[1,"vendor"]
	experiment = data[1,"exp"]
	
	pdf(filename,width=5,height=4,points=12)
	par(mai=c(0.5,0.5,0.4,0.05), mgp = c(1.5,0.5,0))
	
	plot(c(-mxx,mxx),score_range,type='n',main=main,xlab="         Match Distribution        Non-Match Distribution",ylab="Score", axes = FALSE, frame=TRUE)
	at_horz = axTicks(1)
	xrng = mxy - mny
	yrng = mxx - mnx
	#axis(1,at=at_horz,labels=abs(at_horz),lwd=0,lwd.ticks=1)
	axis(2,lwd=0,lwd.ticks=1)
	lines(c(0,0),c(mnx-xrng,mxx+xrng),lty=1,lwd=2,col='black')
	mdata = data[data$match_count > 0,]
	ndata = data[data$nonmatch_count > 0,]
	rect(-0.02*rgx-mdata$m_pdf,mdata$min,-0.02*rgx,mdata$max,col="blue",border="blue",lwd=0.5)
	rect( 0.02*rgx+ndata$n_pdf,ndata$min, 0.02*rgx,ndata$max,col="red",border="red",lwd=0.5)

	dev.off()
}

roc.start = function(filename,main){
	pdf(filename,width=5,height=4,points=12)
	par(mai=c(0.5,0.5,0.4,0.05), mgp = c(1.5,0.5,0))
	
	plot(c(0.001,1.0),c(0,1),type='n',log='x',main=main,xlab="False Accept Rate",ylab="Correct Verification Rate", axes = FALSE, frame=TRUE)
	axis(1,at=c(0.001,0.01,0.1,1),lwd=0,lwd.ticks=1)
	axis(2,at=c(0,0.2,0.4,0.6,0.8,1),lwd=0,lwd.ticks=1)
	
}

roc.plot = function(data,title,col,lty,lwd=2){
	lines(data$far,data$tar,col=col,lty=lty,lwd=lwd)
}

roc.end = function(){
	dev.off()
}

if(file.exists("results/lrpca_gbu_good_hist.csv")){
	data = read.csv("results/lrpca_gbu_good_hist.csv")
	butterfly.plot(data,"LRPCA Good","results/lrpca_gbu_good_hist.pdf",c(-0.2,0.8))
}

if(file.exists("results/lrpca_gbu_bad_hist.csv")){
	data = read.csv("results/lrpca_gbu_bad_hist.csv")
	butterfly.plot(data,"LRPCA Bad","results/lrpca_gbu_bad_hist.pdf",c(-0.2,0.8))
}

if(file.exists("results/lrpca_gbu_ugly_hist.csv")){
	data = read.csv("results/lrpca_gbu_ugly_hist.csv")
	butterfly.plot(data,"LRPCA Ugly","results/lrpca_gbu_ugly_hist.pdf",c(-0.2,0.8))
}

roc.start("results/gbu_roc.pdf","GBU Performance Results")

if(file.exists("results/lrpca_gbu_good_roc.csv")){
	data = read.csv("results/lrpca_gbu_good_roc.csv")
	roc.plot(data,"LRPCA Good","red",1)
}

if(file.exists("results/lrpca_gbu_bad_roc.csv")){
	data = read.csv("results/lrpca_gbu_bad_roc.csv")
	roc.plot(data,"LRPCA Bad","red",2)
}

if(file.exists("results/lrpca_gbu_ugly_roc.csv")){
	data = read.csv("results/lrpca_gbu_ugly_roc.csv")
	roc.plot(data,"LRPCA Ugly","red",3)
}

#legend('bottomright',c("LRPCA Good","LRPCA Bad","LRPCA Ugly"),
#	col=c("red","red","red"),
#	lty=c(1,2,3),
#	lwd=2)

if(file.exists("results/lrpcaocular_gbu_good_roc.csv")){
	data = read.csv("results/lrpcaocular_gbu_good_roc.csv")
	roc.plot(data,"LRPCA Ocular Good","blue",1)
}

if(file.exists("results/lrpcaocular_gbu_bad_roc.csv")){
	data = read.csv("results/lrpcaocular_gbu_bad_roc.csv")
	roc.plot(data,"LRPCA Ocular Bad","blue",2)
}

if(file.exists("results/lrpcaocular_gbu_ugly_roc.csv")){
	data = read.csv("results/lrpcaocular_gbu_ugly_roc.csv")
	roc.plot(data,"LRPCA Ocular Ugly","blue",3)
}

legend('bottomright',c("LRPCA Good","LRPCA Bad","LRPCA Ugly","LRPCA Ocular Good","LRPCA Ocular Bad","LRPCA Ocular Ugly"),
	col=c("red","red","red","blue","blue","blue"),
	lty=c(1,2,3,1,2,3),
	lwd=2)

roc.end()

