

import pyvision as pv
import os.path
import pickle
import facerec2010.baseline.lda as lda
import numpy as np
import copy
#from facerec2010.tools.tune_lrpca import GA_TUNING


TRAINING_SIGSET   = "../../../sigsets/GBU_Training_Uncontrolledx8.xml"
COHORT_SIGSET     = "../../../sigsets/GBU_Training_Uncontrolledx8.xml"
VALIDATION_QUERY  = "../../../sigsets/GBU_500_Query.xml"
VALIDATION_TARGET = "../../../sigsets/GBU_500_Target.xml"
TRAINED_ALGORITHM = "../../../trained/CohortLDA.pkl"
POP_SIZE = 100
N_PROCESSORS = 8



GBU_IMAGE_DIR     = "../../../gbu_reduced/"

GBU_REDUCED_LEYE = pv.Point(128+64,256-20)
GBU_REDUCED_REYE = pv.Point(256+128-64,256-20)
GBU_REDUCED_SIZE = (512,512)





def evaluateAlg(*args,**kwargs):
    #print args
    #print kwargs
    alg = lda.LRLDA(args,**kwargs)

    # Load the sigsets    
    training_set = pv.parseSigSet(TRAINING_SIGSET)
    cohort_set = pv.parseSigSet(COHORT_SIGSET)
    query_set =  pv.parseSigSet(VALIDATION_QUERY)
    target_set =  pv.parseSigSet(VALIDATION_TARGET)
    
    # Read in the training data.
    print "Processing training data."
    for each in training_set:
        label = each[0]
        filename = GBU_IMAGE_DIR + each[1][0]['name'] + '.jpg'
        im = pv.Image(filename)
        alg.addTraining(label,im, None, GBU_REDUCED_LEYE, GBU_REDUCED_REYE)

    # Train the algorithm
    print "Training algorithm."    
    alg.train()
    
    # Process the cohort data
    print "Processing cohort data."
    for each in cohort_set:
        label = each[0]
        filename = GBU_IMAGE_DIR + each[1][0]['name'] + '.jpg'
        im = pv.Image(filename)
        alg.addCohort(im, None, GBU_REDUCED_LEYE, GBU_REDUCED_REYE)

    #Save the GA state.    
    #ilog.pickle(alg, 'LDA_GA')   
    pickle.dump(alg, open(TRAINED_ALGORITHM,'wb'), protocol=2)
    
    # Load the query data
    print "Processing queries"
    queries = []
    for each in query_set:
        label = each[0]
        filename = GBU_IMAGE_DIR + each[1][0]['name'] + '.jpg'
        im = pv.Image(filename)
        fr = alg.getFaceRecord(im, None, GBU_REDUCED_LEYE, GBU_REDUCED_REYE)
        queries.append(fr)

    # Load the target data
    print "Processing targets"
    targets = []
    for each in target_set:
        label = each[0]
        filename = GBU_IMAGE_DIR + each[1][0]['name'] + '.jpg'
        im = pv.Image(filename)
        fr = alg.getFaceRecord(im, None, GBU_REDUCED_LEYE, GBU_REDUCED_REYE)
        targets.append(fr)
    
    # Compute the similarity matrix
    print "Computing similarities."
    mat = alg.similarityMatrix(queries,targets)
    
    bee_mat = pv.BEEDistanceMatrix(mat, os.path.basename(VALIDATION_QUERY), os.path.basename(VALIDATION_TARGET), sigset_dir=os.path.dirname(VALIDATION_QUERY), is_distance=False)
    
    # ROC data is now produced by facerec_score.py
    roc = bee_mat.getROC()
    print roc.results()
    return roc.getFAR(0.001).frr

ilog = pv.ImageLog("/Users/Shared/LDA_GA")

data = pv.Table()
i = 0
#kwargs = copy.deepcopy(lda.I_TUNING)
error = evaluateAlg(*lda.CohortLDA_REGIONS,**lda.CohortLDA_KEYWORDS)
